<?php

    /**
     * functions.php
     *
     * Helper functions.
     */

    require_once(__DIR__ . "/constants.php");

    /**
     * Returns an array of articles for given geography or false on error.
     */
     
    function articles($geo)
    {
        // check cache for articles
        $articles = Cache::get($geo);
        if ($articles !== false)
        {
            return $articles;
        }
        
        // headers for proxy servers
        $headers = [
            "Accept" => "*/*",
            "Connection" => "Keep-Alive",
            "User-Agent" => sprintf("curl/%s", curl_version()["version"])
        ];
    
        // download RSS from Google News
        $context = stream_context_create([
            "http" => [
                "header" => implode(array_map(function($value, $key) { return sprintf("%s: %s\r\n", $key, $value); }, $headers, array_keys($headers))),
                "method" => "GET"
            ]
        ]);
        $contents = @file_get_contents("http://news.google.com/news?output=rss&geo=" . urlencode($geo), false, $context);
        if ($contents === false)
        {
            return false;
        }
    
        // parse RSS
        $rss = @simplexml_load_string($contents);
        @fclose($handle);
        if ($rss === false)
        {
            return false;
        }
    
        // numerically indexed array of articles
        $articles = [];
        
        // iterate over items in channel
        foreach ($rss->channel->item as $item)
        {
            // add article to array
            $articles[] = [
                "link" => (string) $item->link,
                "title" => (string) $item->title
            ];
        }
        
        // cache articles
        Cache::set($geo, $articles);
        
        // return articles
        return $articles;
    }

    /**
     * Executes SQL statement, possibly with parameters, returning
     * an array of all rows in result set or false on (non-fatal) error.
     */
    function query(/* $sql [, ... ] */)
    {
        // SQL statement
        $sql = func_get_arg(0);

        // parameters, if any
        $parameters = array_slice(func_get_args(), 1);

        // try to connect to database
        static $handle;
        if (!isset($handle))
        {
            try
            {
                // connect to database
                $handle = new PDO("mysql:dbname=" . DATABASE . ";host=" . SERVER, USERNAME, PASSWORD);
            }
            catch (Exception $e)
            {
                // trigger (big, orange) error
                trigger_error($e->getMessage(), E_USER_ERROR);
            }
        }

        // ensure number of placeholders matches number of values
        // http://stackoverflow.com/a/22273749
        // https://eval.in/116177
        $pattern = "
            /(?:
            '[^'\\\\]*(?:(?:\\\\.|'')[^'\\\\]*)*'
            | \"[^\"\\\\]*(?:(?:\\\\.|\"\")[^\"\\\\]*)*\"
            | `[^`\\\\]*(?:(?:\\\\.|``)[^`\\\\]*)*`
            )(*SKIP)(*F)| \?
            /x
        ";
        preg_match_all($pattern, $sql, $matches);
        if (count($matches[0]) < count($parameters))
        {
            trigger_error("Too few placeholders in query", E_USER_ERROR);
        }
        else if (count($matches[0]) > count($parameters))
        {
            trigger_error("Too many placeholders in query", E_USER_ERROR);
        }

        // replace placeholders with quoted, escaped strings
        $patterns = [];
        $replacements = [];
        for ($i = 0, $n = count($parameters); $i < $n; $i++)
        {
            array_push($patterns, $pattern);
            array_push($replacements, preg_quote($handle->quote($parameters[$i])));
        }
        $query = preg_replace($patterns, $replacements, $sql, 1);

        // execute query
        $statement = $handle->query($query);
        if ($statement === false)
        {
            trigger_error($handle->errorInfo()[2], E_USER_ERROR);
        }
   
        // if query was SELECT
        // http://stackoverflow.com/a/19794473/5156190
        if ($statement->columnCount() > 0)
        {
            // return result set's rows
            return $statement->fetchAll(PDO::FETCH_ASSOC);
        }

        // if query was DELETE, INSERT, or UPDATE
        else
        {
            // return number of rows affected
            return $statement->rowCount();
        }
    }

?>
